var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-11-25\">\n      <meta name=\"topicid\" content=\"GUID-0D3592DD-2F20-4A41-B19F-544426B8186E\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"indexterm\" content=\"Private and Public Members in Structure Definitions\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Private and Public Members in Structure Definitions</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-0D3592DD-2F20-4A41-B19F-544426B8186E\"></a><div class=\"head\">\n            <h1>Private and Public Members in Structure Definitions </h1>\n         </div>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E875BD\"></a> When defining members (variables and functions) in a struct, the qualifiers <span class=\"code\"> <em class=\"strong\">public</em> </span> and <span class=\"code\"> <em class=\"strong\">private</em> </span> can be used to control the member\'s visibility to users and the Debugger. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E875BF\"></a> <span class=\"code\"> <em class=\"strong\">Public</em> </span> members can be accessed by outside code. If no qualifier is specified, members are\n            considered public by default. In releases prior to <em class=\"strong\">3ds Max 2010</em>, all members were always public. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E875C0\"></a> <span class=\"code\"> <em class=\"strong\">Private</em> </span> members are accessible only from within the structure instance. Attempting to access\n            a private member will result in a Runtime error. If the struct is defined in an encrypted\n            script, the member values will not be shown in the Debugger or in error dumps. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E875C1\"></a>The two supported event handlers <span class=\"code\"> <em class=\"strong\">on create do</em> </span> and <span class=\"code\"> <em class=\"strong\">on clone do</em> </span> are always implicitly private, even if defined in the public section of the struct\n            definition. \n         </p><a name=\"WS73099CC142F48755-2231E4B3128F27532E875C3\"></a><div class=\"table-ruled\">\n            <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n               <colgroup>\n                  <col width=\"100%\">\n               </colgroup>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E875C4\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLES:</em> \n                     </p> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6065\"></a><pre>\nstruct foo2\n(\npublic\nA = 1,\nB = 3,\nfn error = throw \"ZZZ\",\nprivate\nC = 77,\nfn errorPrivate = throw \"ZZZ\",\non create do format \"in create: %\\n\" this,\non clone do format \"in clone: %\\n\" this\n)\n&nbsp;\nf = foo2() --<span class=\"codeComment\">creating calls the \'on create do()\' handler:</span> </pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6064\"></a><pre>\n--&gt; #Struct:foo2(\n--&gt;   error:&lt;fn&gt;; Public,\n--&gt;   c:&lt;data&gt;; Private,\n--&gt;   b:&lt;data&gt;; Public,\n--&gt;   errorPrivate:&lt;fn&gt;; Private,\n--&gt;   a:&lt;data&gt;; Public)\n--&gt; undefined\n--&gt; in create: (foo2 a:1 b:3)\n--&gt; (foo2 a:1 b:3)\n--&gt; OK\n</pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6063\"></a><pre>\nf = foo2 a:1 b:3 c:4 --<span class=\"codeComment\">passing three parameters fails:</span> </pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6062\"></a><pre>\n-- Runtime error: Attempting to access private member: \"c\" in: #Struct:foo2(\n-- b:&lt;data&gt;; Public,\n-- errorPrivate:&lt;fn&gt;; Private,\n-- c:&lt;data&gt;; Private,\n-- error:&lt;fn&gt;; Public,\n-- a:&lt;data&gt;; Public)\n</pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6061\"></a><pre>\nf = foo2 a:10 b:30 --<span class=\"codeComment\">passing two parameters works:</span> </pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6060\"></a><pre>\nin create: (foo2 a:10 b:30)\n(foo2 a:10 b:30)\n</pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-605F\"></a><pre>\nf = foo2 10 30 --<span class=\"codeComment\">passing two arguments works:</span> </pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-605E\"></a><pre>\nin create: (foo2 a:10 b:30)\n(foo2 a:10 b:30)\n</pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-605D\"></a><pre>\nf = foo2 1 3 4 --<span class=\"codeComment\">trying to pass three arguments fails:</span> </pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-605C\"></a><pre>\n-- Runtime error: Attempting to access private member: \"c\" in: #Struct:foo2(\n-- b:&lt;data&gt;; Public,\n-- errorPrivate:&lt;fn&gt;; Private,\n-- c:&lt;data&gt;; Private,\n-- error:&lt;fn&gt;; Public,\n-- a:&lt;data&gt;; Public)\n</pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-605B\"></a><pre>\nf.errorPrivate() --<span class=\"codeComment\">calling the private error() function fails:</span> </pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-605A\"></a><pre>\n-- Runtime error: Attempting to access private member: \"errorPrivate\" in: (foo2 a:10 b:30)\n</pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6059\"></a><pre>\nf.error() --<span class=\"codeComment\">calling the public error() function works:</span> </pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6058\"></a><pre>\n-- Frame:\n-- Runtime error: ZZZ\n</pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6057\"></a><pre>\nf2 = copy f --<span class=\"codeComment\">creating a copy of the existing struct</span> </pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6056\"></a><pre>\nin clone: (foo2 a:10 b:30)\n(foo2 a:10 b:30)\n</pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6055\"></a><pre>\nshowProperties f --<span class=\"codeComment\">the method prints the type of the members</span> </pre></div> \n                  </td>\n               </tr>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-6054\"></a><pre>\na:&lt;data&gt;; Public\nb:&lt;data&gt;; Public\nerror:&lt;fn&gt;; Public\nerrorPrivate:&lt;fn&gt;; Private\nc:&lt;data&gt;; Private\ntrue\n</pre></div> \n                  </td>\n               </tr>\n            </table>\n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E87624\"></a> <span class=\"char_link\"><a href=\"GUID-5FBA4F99-5A57-4B36-9008-F8EC68340AA6.htm\">Structure Definition</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E87625\"></a> <span class=\"char_link\"><a href=\"GUID-35E5A5EF-BF14-4D11-8ABA-CFD9A0E7249E.htm\">Defining Local Functions in Structures</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E87626\"></a> <span class=\"char_link\"><a href=\"GUID-4B31E2CC-8862-4C19-AA07-2DDED173757C.htm\">Structure Scope While Initializing</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E87627\"></a> <span class=\"char_link\"><a href=\"GUID-D5426B3F-7060-4C02-BA75-CB24A66B2ABD.htm\">Structure Inherited Methods</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E87628\"></a> <span class=\"char_link\"><a href=\"GUID-A3A4E597-B4D7-4C38-90A6-E8F78203114E.htm\">Structure Creation and Cloning Event Handlers</a></span> \n                     </p> \n                  </li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";