var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-11-25\">\n      <meta name=\"topicid\" content=\"GUID-0D6E57E4-642E-4115-A4E8-9B2E673371AB\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"indexterm\" content=\"generics\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>dotNet Generics</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-0D6E57E4-642E-4115-A4E8-9B2E673371AB\"></a><div class=\"head\">\n            <h1>dotNet Generics</h1>\n         </div>\n         <p> .NET generic classes and methods can be specialized to specific types in MAXScript\n            by using \n            		<span class=\"code\">Type.MakeGenericType</span> and \n            		<span class=\"code\">MethodInfo.MakeGenericMethod()</span>. These .NET methods create a type-specific version of a type or method. For example,\n            if you run: \n            	 \n         </p>\n         <div class=\"codeBlock\"><pre>callSiteOps = (dotNetClass \"System.Runtime.CompilerServices.CallSiteOps\")\nshowmethods callSiteOps</pre></div>\n         <p>You will get: \n            	 \n         </p>\n         <div class=\"codeBlock\"><pre>.[static]AddRule &lt;CallSite`1&gt;site &lt;T&gt;rule\n.[static]&lt;T&gt;Bind &lt;System.Runtime.CompilerServices.CallSiteBinder&gt;binder &lt;CallSite`1&gt;site &lt;System.Object[]&gt;args\n.[static]ClearMatch &lt;System.Runtime.CompilerServices.CallSite&gt;site\n.[static]&lt;System.Runtime.CompilerServices.CallSite`1[T]&gt;CreateMatchmaker &lt;CallSite`1&gt;site\n.[static]&lt;System.Boolean&gt;Equals &lt;System.Object&gt;objA &lt;System.Object&gt;objB\n.[static]&lt;T[]&gt;GetCachedRules &lt;RuleCache`1&gt;cache\n.[static]&lt;System.Boolean&gt;GetMatch &lt;System.Runtime.CompilerServices.CallSite&gt;site\n.[static]&lt;System.Runtime.CompilerServices.RuleCache`1[T]&gt;GetRuleCache &lt;CallSite`1&gt;site\n.[static]&lt;T[]&gt;GetRules &lt;CallSite`1&gt;site\n.[static]MoveRule &lt;RuleCache`1&gt;cache &lt;T&gt;rule &lt;System.Int32&gt;i\n.[static]&lt;System.Boolean&gt;ReferenceEquals &lt;System.Object&gt;objA &lt;System.Object&gt;objB\n.[static]&lt;System.Boolean&gt;SetNotMatched &lt;System.Runtime.CompilerServices.CallSite&gt;site\n.[static]UpdateRules &lt;CallSite`1&gt;this &lt;System.Int32&gt;matched\ntrue</pre></div>\n         <p>As you can see from the output, the line \n            		<span class=\"code\">[static]AddRule &lt;CallSite`1&gt;site &lt;T&gt;rule</span> indicates that the AddRule(&lt;T&gt;) method is a generic method. If you wanted a MAXScript\n            version of this method that works with System.String, you would first get the method\n            as a dotNetObject, and then specialize it so that &lt;T&gt; is System.String: \n            	 \n         </p>\n         <div class=\"codeBlock\"><pre>addRuleGeneric = getproperty callSiteOps \"AddRule\" asdotnetobject:true\naddRuleStringType = addRuleGeneric.MakeGenericMethod #(dotNetClass \"System.String\")</pre></div>\n         <p> The AddRule() method takes 2 arguments, an instance of a templated class \'CallSite\'\n            and an instance of the specialized class . Looking at the \n            		<a href=\"https://docs.microsoft.com/en-us/dotnet/api/system.runtime.compilerservices.callsiteops?view=netframework-4.8\">CallSite documentation</a>, you will see the method is documented as \n            	 \n         </p>\n         <p><span class=\"code\"> AddRule&lt;T&gt;(CallSite&lt;T&gt;, T)</span> \n         </p>\n         <p> Adds a rule to the cache maintained on the dynamic call site. \n            	 \n         </p>\n         <p>In the MAXScript showmethods output, note the first arg is listed as \'<span class=\"code\">&lt;CallSite`1&gt;site</span>\'. This means that the argument is an instance of a generic class that is templated\n            with a single Type (indicated by the `1 in the class name). Therefore in the documentation\n            it is shown as CallSite&lt;T&gt;. \n            	 \n         </p>\n         <p>If the generic class was templated with two Types, it would be \'2. An example of a\n            class templated with two Types is: \n            	 \n         </p>\n         <p> System.Collections.Generic.Dictionary&lt;TKey,TValue&gt; \n            	 \n         </p>\n         <p>Which in MAXScript looks like: \n            	 \n         </p>\n         <div class=\"codeBlock\"><pre>d = dotnetclass \"System.Collections.Generic.Dictionary`2\"\ndotnet.showConstructors d</pre></div>\n         <p>The showConstructors yields: \n            	 \n         </p>\n         <div class=\"codeBlock\"><pre>System.Collections.Generic.Dictionary`2()\nSystem.Collections.Generic.Dictionary`2 &lt;IDictionary`2&gt;dictionary\nSystem.Collections.Generic.Dictionary`2 &lt;IEqualityComparer`1&gt;comparer\nSystem.Collections.Generic.Dictionary`2 &lt;System.Int32&gt;capacity\nSystem.Collections.Generic.Dictionary`2 &lt;System.Int32&gt;capacity &lt;IEqualityComparer`1&gt;comparer\nSystem.Collections.Generic.Dictionary`2 &lt;IDictionary`2&gt;dictionary &lt;IEqualityComparer`1&gt;comparer</pre></div>\n         <p> By using the following code, you create a instance of a dictionary where \n            		<span class=\"code\">TKey</span> is System.String and \n            		<span class=\"code\">TValue</span> is System.Int32: \n            	 \n         </p>\n         <div class=\"codeBlock\"><pre>genericType = dotnet.GetType \"System.Collections.Generic.Dictionary`2\"\ninnerType1 = dotnet.GetType \"System.String\"\ninnerType2 = dotnet.GetType \"int\"\nspecificType = genericType.MakeGenericType #(innerType1, innerType2)\ntheDict = (dotnetclass \"System.Activator\").CreateInstance specificType</pre></div>\n         <p>This calls constructor: \n            	 \n         </p>\n         <p><span class=\"code\">System.Collections.Generic.Dictionary`2()</span> \n         </p>\n         <p>This code: \n            	 \n         </p>\n         <p><span class=\"code\">theDict = (dotnetclass \"System.Activator\").CreateInstance specificType #(32)</span> \n         </p>\n         <p>Calls the constructor: \n            	 \n         </p>\n         <p><span class=\"code\">System.Collections.Generic.Dictionary`2 &lt;System.Int32&gt;capacity</span> \n         </p>\n         <p> If the constructor took more than one argument, the additional arguments would be\n            specified as additional elements in the array. \n            	 \n         </p>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";