var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-11-25\">\n      <meta name=\"topicid\" content=\"GUID-3EF5C2D1-992C-4F29-8543-72D61F2870D1\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Vector_Map TextureMap and MAXScript</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-3EF5C2D1-992C-4F29-8543-72D61F2870D1\"></a><div class=\"head\">\n            <h1>Vector_Map TextureMap and MAXScript </h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_6F6A278102454599A525D63F2CCAAA7F\"></a> \n            <h2><a name=\"GUID-D428C381-8DDF-4915-AC09-09A95907555F\"></a>Introduction\n            </h2> \n            <p><a name=\"GUID-2081854A-4F00-4E0F-A4F4-5D7F963FA870\"></a>The <em class=\"strong\"> <span class=\"char_link\"><a href=\"GUID-CA5BD5C0-E2B3-4C8F-8115-0E670B71B5E7.htm\">Vector_Map TextureMap</a></span> </em> available in <em class=\"strong\">3ds Max 2014 and higher</em> provides support for scalable vector graphics file formats including the standard\n               SVG XML open standard developed by the World Wide Web Consortium. It also supports\n               compressed SVG in .SVGZ format, Adobe Illustrator .AI files, Adobe Portable Document\n               Format (.PDF) files and Autodesk AutoCAD Pattern (.PAT) files. \n            </p> \n            <p><a name=\"GUID-56898DB9-18B8-4E4C-B867-13017E0F92C5\"></a>In addition to the exposure of the <span class=\"char_link\"><a href=\"GUID-CA5BD5C0-E2B3-4C8F-8115-0E670B71B5E7.htm\">Vector_Map TextureMap</a></span> object to MAXScript documented here, the <em class=\"strong\">3ds Max</em> implementation provides exclusive support for MAXScript functions and expressions\n               inside the actual SVG XML file, allowing for a much tighter integration of vector\n               graphics with the actual scene content. This includes the creation of dynamic vector\n               graphics reacting the scene animation or depending on scene objects properties, text\n               rendering and much more. \n            </p> \n            <p><a name=\"GUID-8F75FF78-8FBC-49F2-9C0C-5033E79CD535\"></a>This topic discusses the creation of SVG files using MAXScript, the inclusion of MAXScript\n               commands inside the SVG XML source files, and provides some examples demonstrating\n               the power of this feature. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_937BDDD05EA848A8B8581166A031BAAB\"></a> \n            <h2><a name=\"GUID-82851FAD-395E-42E7-91F6-E8E3E64DEF6D\"></a>Writing SVG Files\n            </h2> \n            <div class=\"section\"><a name=\"SECTION_59C0F8B0E3C4450FBB91E9C11A9B4C4E\"></a> \n               <h3><a name=\"GUID-06982C1C-BEE9-487B-8340-03C48734C25D\"></a>Basic Format Structure\n               </h3> \n               <p><a name=\"GUID-0557F668-579E-41A2-A414-2B5A3DF4EA6F\"></a>The SVG file is a XML text file. Its content is enclosed in &lt;svg&gt; tags, which identifies\n                  the content as Scalable Vector Graphics definition. \n               </p> \n               <p><a name=\"GUID-0B02C676-4006-415E-AF49-6E724DFE7D71\"></a>Inside the body of the SVG file, one or more entities can be defined, including rectangles,\n                  circles, ellipses, lines, polylines, polygons, images and texts. \n               </p> \n               <p><a name=\"GUID-1E9615D9-10E9-43E3-A531-3BCD70EE3352\"></a>The outside color (stroke color) and inside color (fill color) can be defined as hexadecimal\n                  values. Patterns and Gradients can also be used instead of solid colors. \n               </p> \n               <p><a name=\"GUID-C0D94F03-10C5-4426-9DEC-3D83DE0BA351\"></a>Various properties specific for each entity can be defined, otherwise they will be\n                  assumed to have default values. \n               </p> \n               <p><a name=\"GUID-4B859FEA-FE5C-4315-B1A0-6123C6DB7CF5\"></a>Entities can be transformed and grouped for convenience. \n               </p> \n               <p><a name=\"GUID-CADD59A5-2279-48BE-8A01-2DF37569E1F8\"></a> <a name=\"WS73099CC142F48755-2231E4B3128F27E2F8CF23\"></a><div class=\"table-ruled\">\n                     <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                        <colgroup>\n                           <col width=\"100%\">\n                        </colgroup>\n                        <tr class=\"ruled-odd-row\">\n                           <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27E2F8CF24\"></a><p class=\"table-body\"> <em class=\"strong\">SIMPLEST EXAMPLE:</em> \n                              </p> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-even-row\">\n                           <td class=\"table-body\"> \n                              <div class=\"codeBlock\"><pre>&lt;svg  xmlns=\"http://www.w3.org/2000/svg\"\n      xmlns:xlink=\"http://www.w3.org/1999/xlink\"&gt;\n    &lt;rect x=\"255\" y=\"255\" height=\"512\" width=\"512\"\n          style=\"stroke:#ff8800; fill: #0000aa\"/&gt;\n&lt;/svg&gt;</pre></div> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-odd-row\">\n                           <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27E2F8CF2E\"></a><p class=\"table-body\"> Save this code in a new text file under the name \"Rectangle.svg\" in your 3ds Max\n                                 UserScripts folder. \n                              </p> <a name=\"GUID-576B0642-7363-4D82-B254-BD27420A9256\"></a><p class=\"table-body\">Then create a Vector_Map TextureMap and load this code as the Vector File. </p> <a name=\"GUID-CE966E8C-1941-4D60-A627-2C24EDFDEB67\"></a><p class=\"table-body\">Then render the resulting map into bitmaps with different resolutions - 160x160, 320x320\n                                 and 640x320. \n                              </p> <a name=\"GUID-2FDDC3E7-DC0A-4208-87F6-9E8F6FAA82CE\"></a><p class=\"table-body\">You can do this manually, or using MAXScript: </p> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-even-row\">\n                           <td class=\"table-body\"> \n                              <div class=\"codeBlock\"><pre>theSVGMap = VectorMap()\ntheSVGMap.vectorfile = GetDir #userscripts + \"\\\\rectangle.svg\"\nfor i in #([160,160], [320,320], [640,480]) do \n  display (renderMap theSVGMap size:i filter:true)\n</pre></div> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-odd-row\">\n                           <td class=\"table-body\"> <a name=\"GUID-7B6259D0-DE12-45B9-8A8B-505B291B6D43\"></a><p class=\"table-body\">Evaluating the above MAXScript will render 3 images with different resolutions from\n                                 the same scalable graphics definition. \n                              </p> <a name=\"GUID-A5BF156C-47EC-41CB-8E3D-56DFC4B6D4A9\"></a><p class=\"table-body\">Here are the resulting images: </p> <a name=\"GUID-BD4A345D-13A2-4C29-BCE2-71801CB8DA5B\"></a><p class=\"table-body\"> <img src=\"../images/GUID-2BEAD532-739D-4D92-98C3-D344446A35B4-low.png\"> <img src=\"../images/GUID-ACFB8CDF-C131-4B3E-AD86-C92451244472-low.png\"> \n                              </p> <a name=\"GUID-716B4024-BF09-411E-A67D-44DB5B89D93E\"></a><p class=\"table-body\"> <img src=\"../images/GUID-BBE441AB-CBDC-4E79-BB27-648AB9DB9263-low.png\"> \n                              </p> \n                           </td>\n                        </tr>\n                     </table>\n                  </div> \n               </p> \n               <p><a name=\"GUID-4AB99786-E435-4D33-9CC3-11053344F8ED\"></a>Let\'s discuss the content of the SVG file to understand how it works. \n               </p> \n               <p><a name=\"GUID-1055E6C2-A1CF-4D29-B15B-508AD5091BA5\"></a>The 3ds Max Vector_Map TextureMap assumes that the Canvas the graphics are drawn into\n                  is 1024x1024 units in size. \n               </p> \n               <p><a name=\"GUID-6F21A577-ABA7-417D-B431-283257C051A9\"></a>The origin [0,0] is the upper left corner, just like in MAXScript Bitmap values. The\n                  coordinate of the bottom right corner is [1023,1023] \n               </p> \n               <p><a name=\"GUID-C4EB8ECC-AB87-4EBE-B6F2-D8748D0A7A64\"></a>Thus, when specifying the Position and Size of an entity, we have to assume that the\n                  center of the Canvas is at [511,511]. Specifying the upper left corner of the Rectangle\n                  at x=255 and y=255 and giving it a width and height of 512 units aligns it at the\n                  Canvas\' center. \n               </p> \n               <p><a name=\"GUID-6CE57665-6E39-447E-9A59-2753C62C7480\"></a>This Canvas size assumption is specific for 3ds Max and does not necessarily apply\n                  to SVG and other vector graphics files coming from other sources. That\'s why the Vector_Map\n                  provides Cropping controls to extract only a portion of the rasterized bitmap if the\n                  entities were defined in other coordinates. \n               </p> \n            </div> \n            <div class=\"section\"><a name=\"SECTION_D5AEDCE941E04287B3CE409D277B1173\"></a> \n               <h3><a name=\"GUID-A6C4A9C4-A2CA-4243-A061-356BED86AFEB\"></a>Applying Transformations\n               </h3> \n               <p><a name=\"GUID-6E72CADA-F4EA-466D-849B-F0A1B2F5EC1E\"></a>Let\'s rotate the rectangle that we created at 45 degrees about its center. \n               </p> <a name=\"TABLE_2C1BC554F1884C56A1068A6BF86E419B\"></a><div class=\"table-ruled\">\n                  <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                     <colgroup>\n                        <col width=\"100%\">\n                     </colgroup>\n                     <tr class=\"ruled-odd-row\">\n                        <td class=\"table-body\"> <a name=\"GUID-DA73F98C-BE5B-400D-AF80-384E1A2AD9A8\"></a><p class=\"table-body\"> <em class=\"strong\">TRANSFORMATIONS EXAMPLE:</em> \n                           </p> \n                        </td>\n                     </tr>\n                     <tr class=\"ruled-even-row\">\n                        <td class=\"table-body\"> \n                           <div class=\"codeBlock\"><pre>&lt;svg  xmlns=\"http://www.w3.org/2000/svg\"\n      xmlns:xlink=\"http://www.w3.org/1999/xlink\"&gt;\n    &lt;rect x=\"255\" y=\"255\" height=\"512\" width=\"512\"\n          style=\"stroke:#ff8800; fill: #0000aa\"\n          transform=\"rotate(45 511 511)\" /&gt;\n&lt;/svg&gt;</pre></div> \n                        </td>\n                     </tr>\n                     <tr class=\"ruled-odd-row\">\n                        <td class=\"table-body\"> <a name=\"GUID-16955E69-CB35-407F-9438-B50B0CA0028E\"></a><p class=\"table-body\"> Re-save this code in the same text file under the name \"Rectangle.svg\" in your 3ds\n                              Max UserScripts folder. \n                           </p> <a name=\"GUID-0B7590B2-9C28-419B-BA23-FC045265935B\"></a><p class=\"table-body\">Update the Vector_Map either manually in the Material Editor, or via MAXScript using\n                              <span class=\"code\"> <em class=\"strong\">theSVGMap.reload()</em> </span> \n                           </p> <a name=\"GUID-86D3072A-FA9C-43A3-92A7-F6EEBAA8CF2D\"></a><p class=\"table-body\">Render the resulting map into a bitmap with resolutions of 320x320. </p> \n                        </td>\n                     </tr>\n                     <tr class=\"ruled-even-row\">\n                        <td class=\"table-body\"> \n                           <div class=\"codeBlock\"><pre>theSVGMap.reload()\ndisplay (renderMap theSVGMap size:[320,320] filter:true)</pre></div> \n                        </td>\n                     </tr>\n                     <tr class=\"ruled-odd-row\">\n                        <td class=\"table-body\"> <a name=\"GUID-5CC3EA68-B8C9-4B68-A0A1-ABEB3A72FE32\"></a><p class=\"table-body\">Evaluating this script will render the scalable graphics definition and display the\n                              bitmap. \n                           </p> <a name=\"GUID-688D16A5-3186-4C68-A4E9-FFA8C2CB9138\"></a><p class=\"table-body\">Here are the resulting image: </p> <a name=\"GUID-11CD7301-5A15-4D7C-951D-3CB3F63D5CBF\"></a><p class=\"table-body\"> <img src=\"../images/GUID-E7224255-4D6D-4902-874B-0502CBAEF679-low.png\"> \n                           </p> <a name=\"GUID-012CEE52-A5F7-430C-AFB1-B2054806525E\"></a><p class=\"table-body\">Let\'s also translate the shape 150 units along X before we have rotated it. </p> <a name=\"GUID-48B06A2A-2090-4A45-A30D-3C899B87AF77\"></a><p class=\"table-body\">When multiple transformations are specified, each one modifies the current coordinate\n                              system in the given order: \n                           </p> \n                        </td>\n                     </tr>\n                     <tr class=\"ruled-even-row\">\n                        <td class=\"table-body\"> \n                           <div class=\"codeBlock\"><pre>&lt;svg  xmlns=\"http://www.w3.org/2000/svg\"\n      xmlns:xlink=\"http://www.w3.org/1999/xlink\"&gt;\n    &lt;rect x=\"255\" y=\"255\" height=\"512\" width=\"512\"\n          style=\"stroke:#ff8800; fill: #0000aa\"\n          transform=\"translate(150) rotate(45 511 511)\" /&gt;\n&lt;/svg&gt;</pre></div> \n                        </td>\n                     </tr>\n                     <tr class=\"ruled-odd-row\">\n                        <td class=\"table-body\"> <a name=\"GUID-80A00856-AC5B-4531-A895-F3CB94799606\"></a><p class=\"table-body\">Saving the SVG definition and running the same MAXScript as above produces the following:\n                              \n                           </p> <a name=\"GUID-259E3014-1CD5-4526-A13F-5685AFFB9106\"></a><p class=\"table-body\"> <img src=\"../images/GUID-A2774E00-DEF1-4CA4-B644-2944EA37559C-low.png\"> \n                           </p> <a name=\"GUID-FF8C0BEA-8194-44D1-A36A-F09ED7E196CB\"></a><p class=\"table-body\">In the above case, the translation was applied before the rotation, so the coordinate\n                              system was first shifted to the right, then the rotation was performed. \n                           </p> <a name=\"GUID-60ED3FBF-3D59-4960-81F9-3B95420CB71C\"></a><p class=\"table-body\">If we were to specify the Translation after the rotation, the rotation will occur\n                              first, then the translation will occur in the already rotated coordinate system. \n                           </p> \n                        </td>\n                     </tr>\n                     <tr class=\"ruled-even-row\">\n                        <td class=\"table-body\"> \n                           <div class=\"codeBlock\"><pre>&lt;svg  xmlns=\"http://www.w3.org/2000/svg\"\n      xmlns:xlink=\"http://www.w3.org/1999/xlink\"&gt;\n    &lt;rect x=\"255\" y=\"255\" height=\"512\" width=\"512\"\n          style=\"stroke:#ff8800; fill: #0000aa\"\n          transform=\"rotate(45 511 511) translate(150)\" /&gt;\n&lt;/svg&gt;</pre></div> \n                        </td>\n                     </tr>\n                     <tr class=\"ruled-odd-row\">\n                        <td class=\"table-body\"> <a name=\"GUID-6CD8DBBD-D850-4844-B104-3ECD22983360\"></a><p class=\"table-body\">In this case, the X axis is pointing at the bottom right corner, and the rectangle\n                              is shifted diagonally 150 units in that direction: \n                           </p> <a name=\"GUID-8CB7CC37-F245-4FB6-BA7C-396E57BFF05A\"></a><p class=\"table-body\"> <img src=\"../images/GUID-8BDB0C8E-C1CF-4465-85A1-0ED22E32102F-low.png\"> \n                           </p> \n                        </td>\n                     </tr>\n                  </table>\n               </div> \n            </div> \n            <div class=\"section\"><a name=\"SECTION_BAED295D77DD43E58C7C72C1769030B5\"></a> \n               <h3><a name=\"GUID-214D4B9E-23C5-4DC9-A570-14E31B7D1E61\"></a>Creating Text\n               </h3> \n               <p><a name=\"GUID-6C02CB6B-CDDD-437D-B54D-68EE7104D570\"></a> <a name=\"TABLE_EC03F5C52536499AB25F4D988B85E233\"></a><div class=\"table-ruled\">\n                     <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                        <colgroup>\n                           <col width=\"100%\">\n                        </colgroup>\n                        <tr class=\"ruled-odd-row\">\n                           <td class=\"table-body\"> <a name=\"GUID-9214BBEA-F78E-4A9F-A8A1-37DAD861FC00\"></a><p class=\"table-body\"> <em class=\"strong\"> TEXT EXAMPLE:</em> \n                              </p> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-even-row\">\n                           <td class=\"table-body\"> \n                              <div class=\"codeBlock\"><pre>&lt;svg  xmlns=\"http://www.w3.org/2000/svg\"\n  xmlns:xlink=\"http://www.w3.org/1999/xlink\"&gt;\n  &lt;rect x=\"255\" y=\"255\" height=\"512\" width=\"512\"\n    transform=\"rotate(45 511 511)\"\n    style=\"stroke:#ffff00; fill:#001166\"&gt;\n  &lt;/rect&gt;	 \n  &lt;text x=\"255\" y=\"255\"\n    transform=\"rotate(45 511 511) translate(15 280)\"\n    style=\"stroke:#000000; fill:#008800; font-size:100; font-family:Arial\"&gt;\n    MAXScript\n  &lt;/text&gt;		  \n&lt;/svg&gt;</pre></div> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-odd-row\">\n                           <td class=\"table-body\"> <a name=\"GUID-EA64A5DB-511D-4CC2-AB6E-14660E1784E2\"></a><p class=\"table-body\"> Save this code in a new text file under the name \"RectangleText.svg\" in your 3ds\n                                 Max UserScripts folder. \n                              </p> <a name=\"GUID-9AFA3C36-AF20-4A48-8A12-CDBBE37FD046\"></a><p class=\"table-body\">Run the following MAXScript. </p> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-even-row\">\n                           <td class=\"table-body\"> \n                              <div class=\"codeBlock\"><pre>theSVGMap = VectorMap()\ntheSVGMap.vectorfile = GetDir #userscripts + \"\\\\RectangleText.svg\"\ndisplay (renderMap theSVGMap size:[320,320] filter:true)\n</pre></div> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-odd-row\">\n                           <td class=\"table-body\"> <a name=\"GUID-7F90D0BA-0E30-4393-9BE6-6FD73DFA3C4D\"></a><p class=\"table-body\">Evaluating this script will render the following image: </p> <a name=\"GUID-7622ACB3-C19D-4922-8F08-E5C0BEA303A7\"></a><p class=\"table-body\"> <img src=\"../images/GUID-BF055C8A-C9C2-4024-943D-00F3B1E70BCD-low.png\"> \n                              </p> <a name=\"GUID-D292BA88-51F5-4F04-8C09-CB1285390FE2\"></a><p class=\"table-body\"></p> \n                           </td>\n                        </tr>\n                     </table>\n                  </div> \n               </p> \n            </div> \n            <div class=\"section\"><a name=\"SECTION_7109E863282C4AFE8DAF969DB3DCB19A\"></a> \n               <h3><a name=\"GUID-527CDED5-C8F5-4BF0-B369-06862FEA188E\"></a>Defining Gradients, Stroke Width And Corner Radii\n               </h3> \n               <p><a name=\"GUID-37CE57A4-0DB5-46CB-B12B-A50F50C0E693\"></a>Instead of filling the shape with a solid color, we could define a named Gradient\n                  and use it in place of the color for the rectangle\'s fill: parameter. \n               </p> \n               <p><a name=\"GUID-853043DD-FFB7-4C01-B4F7-670626DF4051\"></a>This Gradient will have the id \"LinearGradient\" and will interpolate linearly along\n                  the Y axis from a dark blue color with zero opacity to a brighter blue color with\n                  full opacity. \n               </p> \n               <p><a name=\"GUID-E8481EAB-7E82-410A-94B1-F30181A35DDF\"></a>In the Rectangle\'s definition, we will use the id \"LinearGradient\" to pass this definition\n                  to the fill: parameter. \n               </p> \n               <p><a name=\"GUID-6F30454C-EC9E-44FC-BA64-96AAB155535E\"></a>We will also set the corner radii to 30 units and the stroke width of both the rectangle\n                  and the text to 3 units. \n               </p> \n               <p><a name=\"GUID-0C4AA404-6C7D-4166-B354-ACFEF071D5A8\"></a> <a name=\"TABLE_D56C604A07A3435D8CB2EB074E6E96DD\"></a><div class=\"table-ruled\">\n                     <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                        <colgroup>\n                           <col width=\"100%\">\n                        </colgroup>\n                        <tr class=\"ruled-odd-row\">\n                           <td class=\"table-body\"> <a name=\"GUID-30DF54D5-E1D7-474F-8864-00AB29C5F592\"></a><p class=\"table-body\"> <em class=\"strong\"> GRADIENT EXAMPLE:</em> \n                              </p> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-even-row\">\n                           <td class=\"table-body\"> \n                              <div class=\"codeBlock\"><pre>&lt;svg  xmlns=\"http://www.w3.org/2000/svg\"\n  xmlns:xlink=\"http://www.w3.org/1999/xlink\"&gt;\n  &lt;defs&gt;\n    &lt;linearGradient id=\"LinearGradient1\"\n      x1=\"0%\" y1=\"0%\"\n      x2=\"0%\" y2=\"100%\"\n      spreadMethod=\"pad\"&gt;\n      &lt;stop offset=\"0%\"   stop-color=\"#001166\" stop-opacity=\"0\"/&gt;\n      &lt;stop offset=\"100%\" stop-color=\"#0055aa\" stop-opacity=\"1\"/&gt;\n    &lt;/linearGradient&gt;\n  &lt;/defs&gt;	  \n  &lt;rect x=\"255\" y=\"255\" height=\"512\" width=\"512\" rx=\"30\" ry=\"30\"\n    transform=\"rotate(45 511 511)\"\n    style=\"stroke:#ffff00; fill:url(#LinearGradient1); stroke-width: 10\"&gt;\n  &lt;/rect&gt;	  \n  &lt;text x=\"255\" y=\"255\" \n    transform=\"rotate(45 511 511) translate(15 280)\"\n    style=\"stroke:#000000; stroke-width:3; fill:#008800; font-size : 100; font-family:Arial\"&gt;\n    MAXScript\n  &lt;/text&gt;		  \n&lt;/svg&gt;</pre></div> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-odd-row\">\n                           <td class=\"table-body\"> <a name=\"GUID-BE0E4767-9364-403E-8C12-D30DCDC203B6\"></a><p class=\"table-body\"> Save this code in a new text file under the name \"RectangleGradientAndText.svg\" in\n                                 your 3ds Max UserScripts folder. \n                              </p> <a name=\"GUID-7E4821CC-41C3-4CC5-8C4A-A95A1E8C2501\"></a><p class=\"table-body\">Run the following MAXScript. We are now rendering into a pre-created bitmap in order\n                                 to produce a correct Alpha channel. \n                              </p> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-even-row\">\n                           <td class=\"table-body\"> \n                              <div class=\"codeBlock\"><pre>theSVGMap = VectorMap alphasource:0 --enable Alpha\ntheSVGMap.vectorfile = GetDir #userscripts + \"\\\\RectangleGradientAndText.svg\"\ntheImage = bitmap 320 320 --create a bitmap with the desired size\nrenderMap theSVGMap into:theImage filter:true --render into that bitmap\ndisplay theImage --display the bitmap\n</pre></div> \n                           </td>\n                        </tr>\n                        <tr class=\"ruled-odd-row\">\n                           <td class=\"table-body\"> <a name=\"GUID-B65541CF-D0EA-4565-BB5D-5EA8F65F39A4\"></a><p class=\"table-body\">Evaluating this script will render the following image, and the alpha channel will\n                                 reflect correctly the gradient\'s varying opacity: \n                              </p> <a name=\"GUID-58A14002-E6C4-428C-8C3C-567FF506DE1C\"></a><p class=\"table-body\"> <img src=\"../images/GUID-59FC9887-A93D-4600-98E6-CED9151EAAD1-low.png\"> <img src=\"../images/GUID-A1877FD3-AA69-4A2F-B871-0575E1BFBA4F-low.png\"> \n                              </p> <a name=\"GUID-339A0CA8-941D-4EE4-A52A-1C29207F3330\"></a><p class=\"table-body\"></p> \n                           </td>\n                        </tr>\n                     </table>\n                  </div> \n               </p> \n            </div> \n            <div class=\"section\"><a name=\"SECTION_A139490AD9B44D26B29856A35E8D3F82\"></a> \n               <h3><a name=\"GUID-271C3F70-52B0-444F-9D19-FF3612809F2E\"></a>Online Documentation\n               </h3> \n               <p><a name=\"GUID-1D2A196A-B5AF-4FE8-9357-7A491A217CFE\"></a>The official website of the SVG format http://www.w3.org/Graphics/SVG/ contains the\n                  current specification. \n               </p> \n               <p><a name=\"GUID-681DBED8-2C91-4781-8384-7A9BF5AAD629\"></a>Several SVG Introductions and Tutorials can be found online. \n               </p> \n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_1A36B71DBCBE4C6DB01D000464C4B7D7\"></a> \n            <h2><a name=\"GUID-0A88CB01-7B46-4FEC-95E0-3111E2F47761\"></a>Generating SVG Files Using MAXScript\n            </h2> \n            <p><a name=\"GUID-1BBBD653-F413-4F00-9E08-3B4D3156555D\"></a>Instead of typing the SVG text files by hand, we can employ MAXScript\'s ability to\n               write ASCII Text Files to create SVG files procedurally. \n            </p> \n            <p><a name=\"GUID-F890490D-18E6-47D5-B653-0984D514EDBE\"></a>For example, let\'s create a function for outputting a circle with specific radius\n               and colors at a user-defined position and call that function multiple times using\n               MAXScript FOR loops. \n            </p> \n            <p><a name=\"GUID-13B9D535-68D8-4707-8507-3D7CB55D4572\"></a>We will also need a function to convert a MAXScript color value to a Hexadecimal color\n               value as used by XML and HTML. \n            </p> \n            <div><a name=\"GUID-AFF7FE1C-9C85-4CE8-B9B0-E51468D7EBD1\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>\'Single quotes\' can be used in place of \"double quotes\" inside the SVG file - this\n                  makes the MAXScript output easier since we don\'t have to escape them as \\\" \n               </div>\n            </div> <a name=\"TABLE_B7C64DDD61B8434EA9EBE9D14A8AD5EC\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-7A6C9B0A-04B2-4E5F-9180-F3EA07A389EC\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>(\n--The following function converts a MAXScript color value to a Hexadecimal color value\nfn ColorToHex col =\n(\n	theComponents = #(bit.intAsHex col.r, bit.intAsHex col.g, bit.intAsHex col.b)\n	theValue = \"#\"\n	for i in theComponents do \n		theValue += (if i.count == 1 then \"0\" else \"\") + i\n	theValue\n)\n\n--The following function outputs a basic circle definition to a given SVG file:\nfn createSVGCircle SVGFile pos radius:100 stroke:yellow fill:red thick:1 =\n(\n	format \"&lt;circle cx=\'%\' cy=\'%\' r=\'%\' \\n\" pos.x pos.y radius to:SVGFile\n	format \"\\tstyle=\'stroke:%; fill:%; stroke-width:%\' &gt;\\n\" (ColorToHex stroke) (ColorToHex fill) thick to:SVGFile\n	format \"&lt;/circle&gt;\\n\" to:SVGFile\n)\n\ntheSVGFilename = GetDir #userScripts + \"\\\\Circles.svg\" --Define an output file name\ntheSVGFile = createFile theSVGFilename --Create a text file\nformat \"&lt;svg xmlns=\\\"http://www.w3.org/2000/svg\\\"\\n\" to:theSVGfile --Output the SVG header lines\nformat \"\\t\\txmlns:xlink=\\\"http://www.w3.org/1999/xlink\\\"&gt;\\n\" to:theSVGfile\n\ntheStep = 100 --This is the step in pixels along both X and Y\nfor y = 0 to 1024 by theStep do --Loop along Y and X with the given Step\n	for x = 0 to 1024 by theStep do --and create circles with position and fill color based on the loops:\n		createSVGCircle theSVGFile [x,y] radius:(theStep/2) stroke:white fill:(color (x/4) (y/4) 100) thick:5\n\nformat \"&lt;/svg&gt;\\n\" to:theSVGfile --End of the SVG body definition \nclose theSVGfile --Close the file\n\ntheSVGMap = VectorMap() --Create a new Vector_Map instance\ntheSVGMap.vectorFile = theSVGFilename --Assign the output file to the new map\ndisplay (renderMap theSVGMap size:[512,512] filter:true) --Render the map at 512 resolution and display\n)</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-8B8B4F8B-1714-4C80-A2B5-1CC08C36FB01\"></a><p class=\"table-body\">Here is the resulting map rendered as image: </p> <img src=\"../images/GUID-C85B1FE6-964D-4221-A095-EDFDD1F8E9EA-low.png\"> <a name=\"GUID-FF70508D-5A3B-4670-9E33-49463BBFD75A\"></a><p class=\"table-body\">By just changing theStep variable from 100 to 50, we can easily produce more and smaller\n                           circles: \n                        </p> <img src=\"../images/GUID-23E33D80-F709-4A54-A407-F2641EC6E6D3-low.png\"> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-E2AB4EA4-C5F6-43E4-BB93-BF2E46CE9626\"></a><p class=\"table-body\">Let\'s modify the loop further - we can shift every second row by half the step to\n                           the side and reduce the Y loop to use half the step. \n                        </p> <a name=\"GUID-32295C6E-01AF-47F7-B754-B0613B80994E\"></a><p class=\"table-body\">We will also use a Stroke Thickness of 2 to produce finer outlines: </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>\n--Partial Code Fragment - insert in the above script!\ntheStep = 50\ncnt = 0 --introduce a counter variable\nfor y = 0 to 1024 by theStep/2 do --divide the vertical step by two\n(\n	cnt = 1-cnt --change the counter between 0 and 1 after each row\n	for x = 0 to 1024 by theStep do --shift the X by half the step multiplied by the counter (0 or 1):\n	(\n		createSVGCircle theSVGFile [x+(theStep/2*cnt),y] radius:(theStep/2) stroke:white fill:(color (x/4) (y/4) 100) thick:2\n	)\n)\n--End Code Fragment</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-D2FB7860-3701-45C7-B6FD-37215EA47F7A\"></a><p class=\"table-body\">The result looks like scales! </p> <img src=\"../images/GUID-1F74973D-77A8-4C4D-B40B-0BC27CB3BD86-low.png\"> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_00A4674D38F74E3486B60C694AFCF084\"></a> \n            <h2><a name=\"GUID-0508B489-30B1-45B0-B13F-144F79C10DAE\"></a>Including Static Scene Properties\n            </h2> \n            <p><a name=\"GUID-AEE9AB77-9239-45D2-AC1C-61FF89CCE2D9\"></a>Since MAXScript can easily read properties of scene objects, we could output scene\n               geometry (for example spline shapes) to SVG and thus turn the 3ds Max viewports into\n               a simple SVG Editor! \n            </p> \n            <p><a name=\"GUID-D99FF2DA-1AFA-47BE-94BC-913248DFE07C\"></a>For example, we can take the Circle drawing function above and implement similar functions\n               for the other typical shapes including Rectangle, Ellipse etc. \n            </p> \n            <p><a name=\"GUID-3BB32F7B-C78F-45DF-A1F1-7C34EFF6D3B2\"></a>Then, we can run through the <em class=\"strong\">3ds Max </em>scene and when we encounter a matching spline primitive, we can grab its properties\n               and output to the SVG file. \n            </p> <a name=\"TABLE_795AD7D7A1154A20A6BE603237135A8B\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-CAAA17E5-59B5-495C-BD5B-DD97D7700992\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>(\nfn ColorToHex col =\n(\n	theComponents = #(bit.intAsHex col.r, bit.intAsHex col.g, bit.intAsHex col.b)\n	theValue = \"#\"\n	for i in theComponents do \n		theValue += (if i.count == 1 then \"0\" else \"\") + i\n	theValue\n)\n\nfn getFillColor obj =\n(\n	if classof obj.material == Standard then\n		obj.material.diffusecolor\n	else \n		obj.wirecolor\n)\n\nfn getPos obj =\n(\n	[obj.pos.x, -obj.pos.y]\n)\n\nfn createSVGCircle SVGFile pos radius:100 stroke:yellow fill:red thick:1 =\n(\n	format \"&lt;circle cx=\'%\' cy=\'%\' r=\'%\' \\n\" pos.x pos.y radius to:SVGFile\n	format \"\\tstyle=\'stroke:%; fill:%; stroke-width:%\' &gt;\\n\" (ColorToHex stroke) (ColorToHex fill) thick to:SVGFile\n	format \"&lt;/circle&gt;\\n\" to:SVGFile\n)\n\nfn createSVGEllipse SVGFile pos rx:100 ry:50 stroke:yellow fill:green thick:1 =\n(\n	format \"&lt;ellipse cx=\'%\' cy=\'%\' rx=\'%\' ry=\'%\'\\n\" pos.x pos.y rx ry to:SVGFile\n	format \"\\tstyle=\'stroke:%; fill:%; stroke-width:%\' &gt;\\n\" (ColorToHex stroke) (ColorToHex fill) thick to:SVGFile\n	format \"&lt;/ellipse&gt;\\n\" to:SVGFile\n)\n\nfn createSVGRect SVGFile pos w:100 h:100 rx:0 stroke:yellow fill:blue thick:1 =\n(\n	format \"&lt;rect x=\'%\' y=\'%\' width=\'%\' height=\'%\' rx=\'%\' ry=\'%\'\\n\" pos.x pos.y w h rx rx to:SVGFile\n	format \"\\tstyle=\'stroke:%; fill:%; stroke-width:%\' &gt;\\n\" (ColorToHex stroke) (ColorToHex fill) thick to:SVGFile\n	format \"&lt;/rect&gt;\\n\" to:SVGFile\n)\n\nfn createSVGText SVGFile pos textString:\"SVG\" fontSize:100 stroke:green fill:orange thick:1 =\n(\n	format \"&lt;text x=\'%\' y=\'%\'\\n\" pos.x pos.y to:SVGFile\n	format \"\\tstyle=\'font-size:%; font-family:Arial; stroke:%; fill:%; stroke-width:%\' &gt;\\n\" fontSize (ColorToHex stroke) (ColorToHex fill) thick to:SVGFile\n	format \"%&lt;/text&gt;\\n\" textString to:SVGFile\n)\n\ntheSVGFilename = GetDir #userScripts + \"\\\\MaxShapes.svg\"\ntheSVGFile = createFile theSVGFilename\nformat \"&lt;svg xmlns=\\\"http://www.w3.org/2000/svg\\\"\\n\" to:theSVGfile\nformat \"\\t\\txmlns:xlink=\\\"http://www.w3.org/1999/xlink\\\"&gt;\\n\" to:theSVGfile\n\nfor obj in objects do\n(\n	local o = obj.baseobject\n	if superclassof o == Shape do\n	(	\n		case classof o of\n		(\n			Circle: createSVGCircle theSVGFile [obj.pos.x,-obj.pos.y] radius:o.radius stroke:obj.wirecolor fill:(getFillColor obj) thick:o.thickness\n			Ellipse: createSVGEllipse theSVGFile [obj.pos.x,-obj.pos.y] rx:(o.width/2) ry:(o.length/2) stroke:obj.wirecolor fill:(getFillColor obj) thick:o.thickness\n			Rectangle: createSVGRect theSVGFile [obj.min.x,-obj.max.y] w:o.width h:o.length rx:o.cornerRadius stroke:obj.wirecolor fill:(getFillColor obj) thick:o.thickness\n			Text: createSVGText theSVGFile [obj.min.x,-obj.min.y] textString:o.text fontSize:o.size stroke:obj.wirecolor fill:(getFillColor obj) thick:o.thickness\n		)\n	)\n)\nformat \"&lt;/svg&gt;\\n\" to:theSVGfile\nclose theSVGfile\n\ntheSVGMap = VectorMap()\ntheSVGMap.vectorFile = theSVGFilename\ndisplay (renderMap theSVGMap size:[512,512] filter:true)\n)</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-9AC9EA27-1AB0-4B06-A70F-C247FECBF2CC\"></a><p class=\"table-body\">Here is a screenshot of a scene containing a green Circle, a dark red Rectangle, a\n                           purple Ellipse and an orange Text shape. \n                        </p> <a name=\"GUID-7A85C798-B017-4720-BE54-6AD3B200A63D\"></a><p class=\"table-body\">The Circle has a Radius of 100 and no material or modifiers. </p> <a name=\"GUID-B9E329B7-A798-4D4C-A421-035A72176AD3\"></a><p class=\"table-body\">The Rectangle is 300x200 units and has Corner Radius of 50; its Renderable property\n                           checked, has a Thickness of 5 and a Standard Material with blue diffuse color assigned.\n                           \n                        </p> <a name=\"GUID-0C532B94-B3A4-4A66-8C1E-3F64C642FFFD\"></a><p class=\"table-body\">The Ellipse with size 200x150 has its Thickness set to 10, a MeshSelect modifier on\n                           the stack, and has a yellow Standard Material assigned. \n                        </p> <a name=\"GUID-47B171FA-AA96-4B0E-95FF-AB2AFAB71D8D\"></a><p class=\"table-body\">The Text has no material and a MeshSelect modifier on the stack. </p> <a name=\"GUID-D8104546-4C68-431B-8D42-7E684CDEAF52\"></a><p class=\"table-body\">These objects were placed in the bottom right quadrant of the World Origin, so that\n                           their Y coordinates are negative, with the origin in the upper left corner. A green\n                           Point Helper shows the position of the 1023,1023 coordinate. \n                        </p> <img src=\"../images/GUID-EA9F58F0-6A4F-4A96-94FF-B89578286132-low.png\"> <a name=\"GUID-014F70A9-7043-4276-B9BD-D5388B85BECA\"></a><p class=\"table-body\">Running the above script produces the following image: </p> <img src=\"../images/GUID-1C836F95-75B3-4499-AFB7-1EF70A74ACD9-low.png\"> <a name=\"GUID-659BB95A-ED41-400B-BDF4-7D1D9F1D5694\"></a><p class=\"table-body\">Note that the order of object creation was reflected in the order of shape drawing.\n                           The stroke color was taken from the wireframe (object) color; the fill color was taken\n                           from the Standard Material\'s diffuse color, or from the wirecolor property when there\n                           was no Material. The Renderable Thickness property was also respected. \n                        </p> <a name=\"GUID-9409853A-3C97-4C16-8410-36C9339C3DA3\"></a><p class=\"table-body\">Also note that the above script would not take rotations into account and would require\n                           significant modifications to support them due to the difference in coordinate systems\n                           and object pivot points between 3ds Max and SVG. \n                        </p> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_67578D5C920B4E98B47D175874EA5B84\"></a> \n            <h2><a name=\"GUID-ED691368-590D-4856-821E-376A75274936\"></a>Setting the SVG Definition Without External File\n            </h2> \n            <p><a name=\"GUID-D4C0C65E-4A73-4BC0-A647-3E811951E826\"></a>The <span class=\"char_link\"><a href=\"GUID-CA5BD5C0-E2B3-4C8F-8115-0E670B71B5E7.htm\">Vector_Map TextureMap</a></span> provides a dedicated method <span class=\"code\"> <em class=\"strong\">.SetSvgString()</em> </span> which can be used to pass the SVG XML definition as a string value without pointing\n               at a file on disk. \n            </p> \n            <p><a name=\"GUID-CE76145A-D55D-44D6-A8AA-7C2BC9AFEDBA\"></a>This means that MAXScript could build a complete SVG description in a String variable\n               or a StringStream and then pass that to the Vector_Map directly. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_D205F74FA0AA4FFD87B85D398B3DCB3E\"></a> \n            <h2><a name=\"GUID-40641CFB-AE54-4D2E-860B-9A3F37F94E11\"></a>Using the MXS Token In SVG Files\n            </h2> \n            <p><a name=\"GUID-7BA44A5C-63B6-41C4-895E-EE7006BC47E4\"></a>MAXScript can not only be used to generate SVG files, the <em class=\"strong\">3ds Max</em> <span class=\"char_link\"><a href=\"GUID-CA5BD5C0-E2B3-4C8F-8115-0E670B71B5E7.htm\">Vector_Map TextureMap</a></span> includes advanced support for MAXScript code embedded in the SVG file itself! \n            </p> \n            <p><a name=\"GUID-4125B960-6FCB-445B-8C3F-87AD32FFB0C8\"></a>The special <em class=\"strong\">mxs</em> token (short for MAXScript) can be used inside the SVG file to include the result\n               of any MAXScript expression in the XML description, as long as it is a String value.\n               \n            </p> \n            <p><a name=\"GUID-E5C952AC-D8B3-468F-87C2-F4162435832C\"></a>This opens the door for much deeper integration of SVG maps with MAXScript and <em class=\"strong\">3ds Max</em>. \n            </p> \n            <p><a name=\"GUID-1532DC4E-25B2-4BEA-A09D-40AA3FB96DDB\"></a>Not only can SVG objects access <em class=\"strong\">3ds Max</em> scene values dynamically, but the Vector_Map will be updated on time changes, allowing\n               for animated maps based on animated scene parameters! \n            </p> <a name=\"TABLE_F687C6B5A77B4902BEC8902BD93D328F\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-11137C6C-14C8-4C47-91B4-00899D7A8616\"></a><p class=\"table-body\"> <em class=\"strong\">ANIMATED SVG EXAMPLE:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>&lt;svg xmlns=\"http://www.w3.org/2000/svg\"\n		xmlns:xlink=\"http://www.w3.org/1999/xlink\"&gt;\n&lt;rect x=\'0.0\' y=\'800.0\' width=\'1024.0\' height=\'100.0\'\n	style=\'stroke:#e4d699; fill:#e4d699; stroke-width:1.0\' &gt;\n&lt;/rect&gt;\n&lt;ellipse cx=\'mxs(at time currentTime $Ellipse001.pos.x as string)\' \n	cy=\'mxs(at time currentTime (-$Ellipse001.pos.y) as string)\' \n	rx=\'mxs(at time currentTime ($Ellipse001.width/2) as string)\' \n	ry=\'mxs(at time currentTime ($Ellipse001.length/2) as string)\'\n	style=\'stroke:#ffffff; fill:#b8e499; stroke-width:3.0\' &gt;\n&lt;/ellipse&gt;\n&lt;/svg&gt;</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-CD95FE87-B59B-467F-B154-8E4890FDF22B\"></a><p class=\"table-body\">Saving the above SVG file and loading it with a Vector_Map will produce an animated\n                           texture following the Position and Radius animation of an Ellipse001 object in the\n                           scene. \n                        </p> <a name=\"GUID-1485AF86-E30E-401C-87DB-8E11A7634736\"></a><p class=\"table-body\">Assigning that Vector_Map texture to the Diffuse channel of a Standard Material and\n                           assigning it to a Plane produces a dynamic real-time rendering \"screen\" in the viewport\n                           showing the same animation. In addition, the Map display in the Material Editor will\n                           also render and show the animation: \n                        </p> <img src=\"../images/GUID-5D1D691F-A033-4F48-A0E4-C804956FA082-low.gif\"> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_6DE7874E265C4D468FDBA6549E97B7F9\"></a> \n            <h2><a name=\"GUID-128D32F7-2C45-42C1-8B82-1CC46BDBEC7B\"></a>Including MAXScript .MS Files In SVG Files\n            </h2> \n            <p><a name=\"GUID-E8161CEA-5E00-438C-B902-CB797107C868\"></a>Instead of using inline MAXScript expressions, the <span class=\"code\"> <em class=\"strong\">mxs()</em> </span> token inside the SVG file can contain function calls to MAXScript functions defined\n               in an external .MS file. This makes the most sense in cases where the calculations\n               are more complex, or the script code has to be modified often and it would be more\n               convenient to separate the MAXScript calculation from the SVG definition. \n            </p> \n            <p><a name=\"GUID-C7EE1C80-3689-4E29-8220-71D1D5F3857A\"></a>The SVG file has to call the MAXScript<span class=\"code\"> <em class=\"strong\">include()</em> </span>function just once with that external .MS file as argument, and any functions defined\n               in it will be accessible to the rest of the SVG body to be called via <span class=\"code\"> <em class=\"strong\">mxs()</em> </span>. These functions must return String values to be included in the SVG stream. \n            </p> \n            <div><a name=\"GUID-D2AEC2FF-03BE-49F9-A9C4-FE2447C5DED6\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>The<span class=\"code\"> <em class=\"strong\">include()</em> </span>call <em class=\"strong\">requires double-quotes</em> for the file name. If the<span class=\"code\"> <em class=\"strong\">include()</em> </span>call has to be inserted within an SVG property value, the SVG value <em class=\"strong\">must</em> be in <em class=\"strong\">single quotes</em> to co-exist with the MAXScript call. \n               </div>\n            </div> <a name=\"TABLE_1BBEC9B68A884A2DB9D7130D5FF08ECE\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-CA628005-FDD8-4176-A5A3-B6270AA15E2A\"></a><p class=\"table-body\"> <em class=\"strong\">MAXSCRIPT INCLUDE FILE:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>fn getUserName =\n(\n	sysinfo.username --this is already a string\n)\nfn getComputerName = \n(\n	sysinfo.computername --this is already a string\n)\nfn getNumCPUs =\n(\n	sysinfo.cpucount as string --this is an Integer, so needs conversion to string\n)</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-9FA8C53C-F215-4EB8-848E-F559DF6930E4\"></a><p class=\"table-body\">Save the above code to \"TextIncludeFn.ms\" in your UserScripts folder (getDir #userscripts)\n                           \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-5DC6C4A7-6923-4386-B31C-8F0FE7F9A3DC\"></a><p class=\"table-body\"> <em class=\"strong\">SVG FILE:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>&lt;svg  xmlns=\"http://www.w3.org/2000/svg\"\n  xmlns:xlink=\"http://www.w3.org/1999/xlink\"&gt;\n  &lt;text x=\'100\' y=\'300\'\n    style=\'stroke:#00FF88; fill:#005588; font-size:70; font-family:Arial\'&gt;\n    Hello mxs(include \"TextIncludeFn.ms\" getUserName())!\n  &lt;/text&gt;		  \n  &lt;text x=\'100\' y=\'400\'\n    style=\'stroke:#00FF88; fill:#005588; font-size:70; font-family:Arial\'&gt;\n    Your Computer mxs(getComputerName())\n  &lt;/text&gt;		  \n  &lt;text x=\'100\' y=\'500\'\n    style=\'stroke:#00FF88; fill:#005588; font-size:70; font-family:Arial\'&gt;\n    has mxs(getNumCPUs()) CPUs\n  &lt;/text&gt;		  \n&lt;/svg&gt;</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-DA575F04-1A6F-46E0-81C0-64661F158445\"></a><p class=\"table-body\">Save the above SVG definition as \"TextIncludeFn.svg\" in the same folder as the .MS\n                           file - (getDir #userscripts) \n                        </p> <a name=\"GUID-9B3CF2AA-CAEA-4E08-9C6F-6C597655C305\"></a><p class=\"table-body\">Note that the include() call is performed just once before the first use of getUserName(),\n                           all further mxs function calls don\'t need the .MS file re-included! \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-2CD8264F-559F-49BC-94EB-03B298E74379\"></a><p class=\"table-body\"> <em class=\"strong\">CREATE VECTORMAP TEXTUREMAP:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>theVMap = VectorMap() --create a vector map\ntheSVGFileName = GetDir #userScripts + \"\\\\TextIncludeFn.svg\" --define the file name of the SVG map source\ntheVMap.vectorfile = theSVGFileName --assign the SVG file to the map\ntheTextImage = bitmap 512 512 --create an image for the text\nrenderMap theVMap into:theTextImage filter:true --render the Vector Map into the image\ndisplay theTextImage --display the resulting image</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-9FB386F9-D0F3-4594-BDBD-6553C44B117A\"></a><p class=\"table-body\"> <em class=\"strong\">RESULT EXAMPLE:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\">&nbsp; <img src=\"../images/GUID-23DC48D7-904C-41C4-92C4-84C10055D83D-low.png\"> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_B4BA154CAAE74E8797051AAEF4710078\"></a> \n            <h2><a name=\"GUID-4C31015E-036F-434C-B445-895BA900F4E9\"></a>Using SVG To Render Text To Bitmap\n            </h2> \n            <p><a name=\"GUID-53B41930-0D1B-4243-8FC0-405EE9300463\"></a>In the following example, we will use the text rendering capabilities of the <span class=\"char_link\"><a href=\"GUID-CA5BD5C0-E2B3-4C8F-8115-0E670B71B5E7.htm\">Vector_Map TextureMap</a></span> to generate a bitmap containing render time information. \n            </p> \n            <p><a name=\"GUID-76FDB72D-7B7D-41E2-B338-00E1006F757B\"></a>We will create a bitmap using the <span class=\"code\"> <em class=\"strong\">renderMap()</em> </span> function and overlay it on top of an image produced using the <span class=\"code\"> <em class=\"strong\">render()</em> </span> function. \n            </p> <a name=\"TABLE_FE0B0FD259354497B7FF1D646D7CB115\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-8D74DE88-7E30-4939-9DFB-EBA40744D461\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><pre>renderW = 640 --define the render output width\nrenderH = 480 --define the render output height\ntextSize = 20 --define the overlay text\'s height in pixels\ntheImage = bitmap renderW renderH --define a bitmap for the render output\nst = timestamp() --get a timestamp before rendering\nrender to:theImage vfb:off --render the scene into the existing bitmap\net = timestamp() --get a time stamp after rendering\ntheTime = \"Image Render Time: \" +((et-st)/1000.0) as string + \" seconds.\" --define the text to overlay\ntheVMap = VectorMap() --create a vector map\ntheSVGFileName = GetDir #userScripts + \"\\\\RenderInfoMap.svg\" --define the file name of the SVG map source\ntheSVGFileHandle = createFile theSVGFileName --create a new text file with that name\nformat \"&lt;svg xmlns=\\\"http://www.w3.org/2000/svg\\\"\\n\" to:theSVGFileHandle --define the svg header\nformat \"\\txmlns:xlink=\\\"http://www.w3.org/1999/xlink\\\"&gt;\\n\" to:theSVGFileHandle\nformat \"\\t&lt;text x=\'0\' y=\'%\'\\n\" textSize to:theSVGFileHandle --define the text position\nformat \"\\t\\tstyle=\'font-size : %;\\n\" textSize to:theSVGFileHandle --define the text font size\nformat \"\\t\\tfont-family: Arial; \\n\" to:theSVGFileHandle --define the text font family\n--format \"\\t\\tstroke: #ffffff;\\n\" to:theSVGFileHandle --define the text outline. Bad Idea, makes it too thick\nformat \"\\t\\tfill: #ffffff\\n\" to:theSVGFileHandle --define the fill color of the text as white\nformat \"\\t\\t\'\\n\" to:theSVGFileHandle --close the font style definition\nformat \"\\t&gt;%&lt;/text&gt;\\n\" theTime to:theSVGFileHandle --output the text to display\nformat \"&lt;/svg&gt;\\n\" to:theSVGFileHandle --end the SVG definition\nclose theSVGFileHandle --close the SVG file\ntheVMap.vectorfile = theSVGFileName --assign the SVG file to the map\ntheVMap.alphasource = 0 --enable Alpha output\ntheTextImage = bitmap 1000 1000 --create an image for the text\nrenderMap theVMap into:theTextImage filter:true --render the Vector Map into the image\n--composite the text from the image onto the rendered image\npasteBitmap theTextImage theImage (box2 0 0 1000 (textSize*2.0)) [5,renderH-(textSize*1.5)] type:#composite\ndisplay theImage --display the result</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"GUID-458E6D2F-47E0-4C46-9AF7-2514AC7C962D\"></a><p class=\"table-body\"> <em class=\"strong\">RESULT EXAMPLE:</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\">&nbsp; <img src=\"../images/GUID-373A756F-BDD6-43CC-917B-34A95FBC8083-low.png\"> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_7D25AECB29D44759A2D9DA70AB5298DA\"></a> \n            <h2><a name=\"GUID-7310F848-0787-4AE6-B612-3081D6E125D7\"></a>Learn More About SVG and Vector_Map\n            </h2> \n            <p><a name=\"GUID-6C27F14C-86A3-42E4-AC1B-9E3E106B4F79\"></a>A three-part tutorial in the How To... section demonstrates the use of the Vector_Map\n               and the SVG format to render geometry objects as polygons: \n            </p> <a name=\"UL_F2F055E49E7243CAB79C3230568CC14D\"></a><ul>\n               <li><span class=\"char_link\"><a href=\"GUID-6B5EDC11-A154-4AA7-A972-A11AC36949E9.htm\">How To ... Develop An SVG Polygon Renderer - Part 1</a></span> - Basic Flat Rendering \n               </li>\n               <li><span class=\"char_link\"><a href=\"GUID-0453DFB1-1773-40DC-928E-C328D53EE293.htm\">How To ... Develop An SVG Polygon Renderer - Part 2</a></span> - Back-To-Front Sorted Rendering \n               </li>\n               <li><span class=\"char_link\"><a href=\"GUID-14B02950-26DF-498C-A88A-CD87DC84473C.htm\">How To ... Develop An SVG Polygon Renderer - Part 3</a></span> - Facets with Blinn Lighting and Shading \n               </li>\n            </ul> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";