var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-11-25\">\n      <meta name=\"topicid\" content=\"GUID-608825CD-D025-4054-91E2-2562A6AB31CF\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"indexterm\" content=\"rollout\">\n      <meta name=\"indexterm\" content=\"visibility of locals functions structures and user-interface items in rollout code\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Visibility of Locals, Functions, Structures and User-Interface Items in Rollout Code</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-608825CD-D025-4054-91E2-2562A6AB31CF\"></a><div class=\"head\">\n            <h1>Visibility of Locals, Functions, Structures and User-Interface Items in Rollout Code\n               \n            </h1>\n         </div>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-6815\"></a>The ordering of user-interface items and functions and sub-rollouts in a utility is\n            important when referencing other items and rollouts in handler or local function code.\n            Specifically, you can only reference a user-interface item or rollout or function\n            in a handler or other function if that item or rollout or function has been defined\n            earlier in the utility. \n         </p>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-6814\"></a>A recommended ordering can help ensure this: \n         </p><a name=\"OL_D42CD144568E4500AEE6A275DC043703\"></a><ol type=\"1\" start=\"1\">\n            <li>local variables </li>\n            <li>structs </li>\n            <li>user-interface items </li>\n            <li>nested rollouts </li>\n            <li>functions </li>\n            <li>event handlers </li>\n         </ol>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-680D\"></a>In some situations, cross-referencing means there\'s no way to order definitions and\n            still pre-define everything. To remedy this, MAXScript lets you pre-declare local\n            rollouts and functions, so code that comes before the function or rollout definition\n            will see the correct object. You do this by declaring such rollouts and functions\n            as uninitialized locals. In the following example, there are two rollouts, <span class=\"code\"> <em class=\"strong\">ro1</em> </span> and <span class=\"code\"> <em class=\"strong\">ro2</em> </span>, that each want to reference items within the other. By pre-declaring them as locals\n            in the main utility, this cross-referencing is possible: \n         </p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-680C\"></a><div class=\"table-ruled\">\n            <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n               <colgroup>\n                  <col width=\"100%\">\n               </colgroup>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-680B\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE:</em> \n                     </p> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-748A\"></a><pre>\n(\nlocal ro1, ro2 -- pre-declare local rollouts\nrollout ro1 \"Rollout 01\"\n(\ncheckbox chk_checkbox \"Check The Box In Rollout 02\" enabled:false\non chk_checkbox changed state do ro2.chk_checkbox.checked = state\n)\nrollout ro2 \"Rollout 02\"\n(\ncheckbox chk_checkbox \"Enable The Checkbox In Rollout 01\"\non chk_checkbox changed state do ro1.chk_checkbox.enabled = state\n)\ntheFloater = newRolloutFloater \"Test Local Visibility\" 250 110\naddRollout ro1 theFloater\naddRollout ro2 theFloater\n) --end script\n</pre></div> \n                  </td>\n               </tr>\n            </table>\n         </div>\n         <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-67F5\"></a>As a general rule, it is recommended that all local variable names be explicitly declared\n            as locals at their outermost scope. This will prevent any conflicts from occurring\n            if another script has declared the same variable name as a global variable. While\n            functions, structures definitions, and rollouts will always be local to the utility\n            they are defined in (even if they have the same name as a global variable), declaring\n            these as local will ensure the variables are defined, even if the order in which they\n            are defined is changed. Here\'s an example showing these declarations: \n         </p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-67F3\"></a><div class=\"table-ruled\">\n            <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n               <colgroup>\n                  <col width=\"100%\">\n               </colgroup>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-67F2\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE:</em> \n                     </p> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-7489\"></a><pre>\nutility foo \"Object Frabulator\"\n(\n-- local variables\nlocal target_obj\n-- local functions\nlocal prop_name\n-- local rollouts\nlocal setup\nfn prop_name obj name =\nfor c in obj.children do c.name = name + \"_\" + c.name\ncheckbutton setup_btn \"Setup\"\nedittext name_box \"New name:\"\n-- use a checkbutton to dynamically control presence of panels\non setup_btn changed state do\nif state then addRollout setup else removeRollout setup\non foo close do\nremoveRollout setup -- always close rollouts\nrollout setup \"Setup fraber\" -- local panel\n(\nlabel hello\npickbutton pick_tgt \"Pick object\"\non pick_tgt picked obj do\n(\ntarget_obj = obj -- access utility local\nname_box.text = obj.name -- access utility item\nprop_name obj obj.name -- call utility local fn\n)\n)\n)\n</pre></div> \n                  </td>\n               </tr>\n            </table>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";