var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-11-25\">\n      <meta name=\"topicid\" content=\"GUID-99E3B658-D86C-4FD6-9B37-753AEF05FB74\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"indexterm\" content=\"DotNet Objects and Classes Life Time Control\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>DotNet Objects and Classes Life Time Control</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-99E3B658-D86C-4FD6-9B37-753AEF05FB74\"></a><div class=\"head\">\n            <h1>DotNet Objects and Classes Life Time Control </h1>\n         </div>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-105D\"></a> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-105C\"></a> In versions prior to <em class=\"strong\">3ds Max 2010</em>, performing MAXScript garbage collection can delete the DotNet event handlers too\n               early. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-105A\"></a>With this method available in <em class=\"strong\">3ds Max 2010 and higher</em>, this can be avoided by setting the life time control of the corresponding objects\n               to be handled by DotNet instead of MAXScript. \n            </p> \n            <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-67A0\"></a><pre><em class=\"codeEmphasisStrong\">dotNet.setLifetimeControl</em> {&lt;dotNetObject&gt; | &lt;dotNetClass&gt;} {<em class=\"codeEmphasisStrong\">#mxs</em> | <em class=\"codeEmphasisStrong\">#dotnet</em>}</pre></div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1057\"></a>Controls whether the MAXScript value wrapping the DotNet object or DotNetclass can\n               be garbage collected when there are no MAXScript references to the value (<span class=\"code\"> <em class=\"strong\">#mxs</em> </span>), or whether the DotNet object is destroyed before the MAXScript value can be garbage\n               collected (<span class=\"code\"> <em class=\"strong\">#dotnet</em> </span>). \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1056\"></a>By default, a dotNetObject value holds a <em class=\"strong\">strong reference</em> to the DotNet object, and the DotNet object is deleted only when the dotNetObject\n               value is deleted. This is equivalent to specifying <span class=\"code\"> <em class=\"strong\">#mxs</em> </span> as second argument in the above method. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1055\"></a>If you specify <span class=\"code\"> <em class=\"strong\">#dotnet</em> </span> as the second argument, the dotNetObject cannot be collected until the DotNet object\n               is deleted. The dotNetObject value holds a weak reference to the DotNet object and\n               the DotNet object is deleted when no other DotNet objects hold a reference to it and\n               a DotNet garbage collection occurs. \n            </p> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1053\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1052\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE 1 - HANDLERS DELETED</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-679F\"></a><pre>\n(\nfunction whenButtonIsPressed =\n(\nprint \"Pressed from a .NET button\"\n)\nfunction formIsClosed =\n(\nprint \"form closed\"\n)\nlocal mButton = dotNetObject \"System.windows.forms.button\"\nmButton.text = \".NET Button\"\nmButton.size = dotNetObject \"System.Drawing.Size\" 100 100\nmButton.location = dotNetObject \"System.Drawing.Point\" 50 50\nlocal hForm = dotNetObject \"System.Windows.Forms.Form\"\nhForm.controls.add mButton\ndotNet.addEventHandler mButton \"click\" whenButtonIsPressed\ndotNet.addEventHandler hForm \"Closed\" formIsClosed\nhForm.show()\nok\n)\ngc()\n</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1035\"></a><p class=\"table-body\">Evaluating the above code creates a DotNet Form with a DotNet button and two event\n                           handlers - one to print a message to the Listener when the button is pressed and one\n                           to print a message when the Form is closed. \n                        </p> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1034\"></a><p class=\"table-body\">Because of the MAXScript Garbage Collection performed after the creation of the Form,\n                           both handlers are deleted immediately and nothing is printed when pressing the button\n                           or closing the form. \n                        </p> \n                     </td>\n                  </tr>\n               </table>\n            </div> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1032\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1031\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE2 - HANDLERS PRESERVED</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-1030\"></a><p class=\"table-body\">By switching the life time control of the button and the form to DotNet, it is ensured\n                           that the event handlers of both buttons are preserved after a MAXScript Garbage Collection:\n                           \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-679E\"></a><pre>\n(\nfunction whenButtonIsPressed =\n(\nprint \"Pressed from a .NET button\"\n)\nfunction formIsClosed =\n(\nprint \"form closed\"\n)\nlocal mButton = dotNetObject \"System.windows.forms.button\"\nmButton.text = \".NET Button\"\nmButton.size = dotNetObject \"System.Drawing.Size\" 100 100\nmButton.location = dotNetObject \"System.Drawing.Point\"50 50\nlocal hForm = dotNetObject \"System.Windows.Forms.Form\"\nhForm.controls.add mButton\ndotNet.addEventHandler mButton \"click\" whenButtonIsPressed\ndotNet.addEventHandler hForm \"Closed\" formIsClosed\ndotNet.setLifetimeControl mButton #dotnet\ndotNet.setLifetimeControl hForm #dotnet\nhForm.show()\nok\n)\ngc()\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-100F\"></a> <span class=\"char_link\"><a href=\"GUID-779FD7AC-953D-4567-B2A8-60B1D8695B95.htm\">DotNet In MAXScript</a></span> \n                     </p> \n                  </li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";