var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-11-25\">\n      <meta name=\"topicid\" content=\"GUID-B8218069-D360-4139-AB8A-FB78292A0839\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"indexterm\" content=\"working with the NURBS classes\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Working with the NURBS Classes</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-B8218069-D360-4139-AB8A-FB78292A0839\"></a><div class=\"head\">\n            <h1>Working with the NURBS Classes </h1>\n         </div>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4915\"></a> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4914\"></a> Unlike other parts of MAXScript that let you script <em class=\"strong\">3ds Max</em> scene objects directly, the NURBS family of classes provides an indirect means of\n               working with <em class=\"strong\">3ds Max</em> NURBS objects. They allow you to build an idealized structure representing <em class=\"strong\">3ds Max</em> NURBS objects and then use this structure to either create new or add to scene NURBS\n               objects or to access certain parts of existing scene objects.\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4912\"></a>The structure is made up of instances of the MAXScript NURBS classes, listed in <span class=\"char_link\"><a href=\"GUID-75CD4DE9-8024-4E25-B147-0A0EC8B10031.htm\">NURBS Classes</a></span>. The main class for this is <span class=\"code\"> <em class=\"strong\">NURBSSet</em> </span>. It is the structure that corresponds to a scene NURBS object and contains a collection\n               of NURBS items such as curves, surfaces, dependent surfaces, points, and so on, which\n               are themselves instances of the other MAXScript NURBS classes. You construct a NURBSSet\n               object describing a NURBS object and then have it be instantiated as a real scene\n               object. Conversely, you can ask for a NURBSSet to be created that corresponds to a\n               given scene NURBS object and use it to examine the component NURBS items and to modify\n               certain properties directly on the scene object it describes.\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4911\"></a> <em class=\"strong\">Parameter Validity</em> \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4910\"></a>The <em class=\"strong\">3ds Max</em> NURBS SDK methods and the corresponding MAXScript methods and properties perform\n               very little parameter validation. As a result, passing invalid parameters to the functions\n               or accessing invalid ranges will generate Assertion Failures easily. In most cases\n               <em class=\"strong\">3ds Max</em> will not crash if Retry or Ignore is selected. An example of this is:\n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27EE529-490F\"></a><ol type=\"1\" start=\"1\">\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-490D\"></a>Create a CV surface and select it.\n                  </p> \n               </li>\n               <li> \n                  <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-490B\"></a>Enter in MAXScript:\n                  </p> \n               </li>\n            </ol> \n            <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-22D0\"></a><pre>\ns = getnurbsset $ #relational\nobj = getobject s 1\ntess = getviewtess obj #displacement\n</pre></div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4907\"></a>This is invalid because the viewport does not support displacement approximation,\n               only the production renderer does. Therefore, this will generate an Assertion Failure.\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4906\"></a> <em class=\"strong\">NURBSId\'s and Indexes</em> \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4905\"></a>An important concept to understand when working with NURBSSets and the other NURBS\n               classes, is the way you identify component items in the set. Before a NURBSet is instantiated\n               as a scene NURBS object, the individual curves and points and surfaces are identified\n               by an index number. As you append objects to the set they are assigned successive\n               indexes, starting at 1. When you want to refer to a particular item, for example to\n               specify the parent surfaces in a blend surface, you use this index. Once an object\n               has been added to a NURBSSet, its index can determined through the <span class=\"code\"> <em class=\"strong\">.index</em> </span> property. This is a read-only property.\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4904\"></a>However, once a NURBSet is associated with a scene object, either by being instantiated\n               or because it was directly created from an existing scene object, this index is <em class=\"strong\">no longer valid</em>. Instead, a different number, called a <span class=\"code\"> <em class=\"strong\">NURBSId</em> </span>, is assigned to each item and you must use that Id when referring to items from then\n               on. The NURBSId of any item in an instantiated NURBSSet can be obtained through the\n               <span class=\"code\"> <em class=\"strong\">.NURBSId</em> </span> property on that object.\n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27EE529-4903\"></a>These identifiers, indexes or NURBSId\'s, are used in many places to specify the parent\n               objects for other dependent objects. Often, these are specified using the <span class=\"code\"> <em class=\"strong\">.parent</em> </span> or <span class=\"code\"> <em class=\"strong\">.parentID</em> </span> properties on a dependent object. You must be sure to use the correct property depending\n               on whether the containing NURBSSet has been associated with a scene object or not:\n               <span class=\"code\"> <em class=\"strong\">.parent</em> </span> for non-associated NURBSSets, <span class=\"code\"> <em class=\"strong\">.parentID</em> </span> for associated NURBSSets. Remember that indexes used in <span class=\"code\"> <em class=\"strong\">.parent</em> </span> settings are 1-based.\n            </p> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";