var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-11-25\">\n      <meta name=\"topicid\" content=\"GUID-BA683828-589C-4BBB-897C-FD2B7DA0E147\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Non-Local Variables As Captions Of Rollouts and UI Controls</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-BA683828-589C-4BBB-897C-FD2B7DA0E147\"></a><div class=\"head\">\n            <h1> Non-Local Variables As Captions Of Rollouts and UI Controls </h1>\n         </div>\n         <p><a name=\"GUID-4941CA93-5BC2-4D11-BDC9-24613789DF93\"></a>In versions prior to <em class=\"strong\">3ds Max 2012</em>, the caption parameter of a rollout or UI control is provided as a string literal\n            and cannot be specified using a variable name. This made it difficult to use computed\n            names as it required the setting of the rollout\'s or control\'s caption property after\n            its construction, or the creation of the whole rollout using string building and evaluation\n            or the Rollout Creator functions. \n         </p>\n         <p><a name=\"GUID-C372CA50-AD2D-4506-B7F2-3512C116053B\"></a> <em class=\"strong\"> 3ds Max 2012</em> added the ability to specify captions of rollouts and UI controls uisng non-local\n            variable name. This means that the variable cannot be a local variable or a variable\n            name passed through as a function parameter. The variable must be either in the global\n            scope or in the top-level scope such as, a structure definition, MacroScript definition,\n            or scripted plugin definition. \n         </p>\n         <p><a name=\"GUID-E64BB9EA-89E3-4692-941E-E3D9D9A92D75\"></a> This restriction stems from the fact that a rollout definition is a <em class=\"strong\">compile-time construct</em>, and the local variables are defined using the context when the rollout is compiled.\n            However, an evaluation of the variables is attempted when the rollout is created,\n            by which time it is no longer in the compile context. So, if a local variable is used\n            as the caption, a compile time error will be thrown: \n         </p><a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-6722\"></a><div class=\"table-ruled\">\n            <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n               <colgroup>\n                  <col width=\"100%\">\n               </colgroup>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128E0E4A0FC-6721\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE</em> \n                     </p> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-7472\"></a><pre>\nfn test cap1 =\n(\n  cap2 = \"cap2\"\n  Rollout test cap1\n  (\n    Spinner s1 cap2\n  )\n  createdialog test\n)\n-- Compile error: No outer or current scope local variable references permitted here: cap1\n-- In line: Rollout test cap1\n</pre></div> \n                  </td>\n               </tr>\n            </table>\n         </div>\n         <p><a name=\"GUID-F24628C0-8C35-40C2-B568-DB1D62F57B8F\"></a> If you change the above usage of cap1 to a string literal, you will get another error\n            on the usage of cap2. \n         </p>\n         <p><a name=\"GUID-6809B3A6-B754-4B74-B7C8-DF76161A3E94\"></a>The following shows various usages that compile successfully: \n         </p><a name=\"TABLE_090A8804B6994085A1A99C5DB8F28F38\"></a><div class=\"table-ruled\">\n            <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n               <colgroup>\n                  <col width=\"100%\">\n               </colgroup>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> <a name=\"GUID-120A4922-3F67-486F-BD52-BED431D835DD\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE</em> \n                     </p> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"CODEBLOCK_81083806EF584AC2BAE7D838E9205408\"></a><pre>\ncap1 = \"cap1\"\ncap2 = \"cap2\"\nfn test =\n(\n  Rollout test cap1\n  (\n    Spinner s1 cap2\n  )\n  createdialog test\n)\ntest()\n</pre></div> \n                  </td>\n               </tr>\n            </table>\n         </div><a name=\"TABLE_E57CCE7854D84BAB97122A637A498511\"></a><div class=\"table-ruled\">\n            <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n               <colgroup>\n                  <col width=\"100%\">\n               </colgroup>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> <a name=\"GUID-AEB1E7D4-02A2-49C3-B0AC-650F63CDCB16\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE</em> \n                     </p> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"CODEBLOCK_B5BC951ED7D5479684EE787ACACFDB03\"></a><pre>\nstruct xxx\n(\n  cap1b = \"cap1\",\n  cap2b = \"cap2\",\n  fn createRollout = \n  (\n    rollout test cap1b\n    (\n      spinner s00\n      spinner s01 \"A\"\n      spinner s1 cap2b\n    )\n    createDialog test\n  )\n)\nx = xxx()\nr=x.createRollout()\n</pre></div> \n                  </td>\n               </tr>\n            </table>\n         </div><a name=\"TABLE_9D573863289745378535154DB0641F65\"></a><div class=\"table-ruled\">\n            <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n               <colgroup>\n                  <col width=\"100%\">\n               </colgroup>\n               <tr class=\"ruled-odd-row\">\n                  <td class=\"table-body\"> <a name=\"GUID-2DC86432-BE0A-4F31-A474-E7EBB2C6E8B1\"></a><p class=\"table-body\"> <em class=\"strong\">EXAMPLE</em> \n                     </p> \n                  </td>\n               </tr>\n               <tr class=\"ruled-even-row\">\n                  <td class=\"table-body\"> \n                     <div class=\"codeBlock\"><a name=\"CODEBLOCK_402B4B5E4F434EE8817268DE484B0123\"></a><pre>\nplugin material MyMaterial\nname:\"MyMaterial\"\nclassID:#(0x69fedc0d, 0x7c79a4d2)\nextends:Standard replaceUI:true \n(\n  local rollout_title = \"Hardware Shaders\" \n  local spinner_title = \"AA\"\n  parameters hardwareShaders rollout:shaderRoll\n  (\n  )\n  rollout shaderRoll rollout_title width:328 height:189\n  (\n    spinner s1 spinner_title\n  )\n)\na=mymaterial()\nmeditmaterials[1] = a\n</pre></div> \n                  </td>\n               </tr>\n            </table>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";