var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"3DSMAX\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2019-11-25\">\n      <meta name=\"topicid\" content=\"GUID-FAF8B2F4-3199-4757-A604-49E1F5BBEBE9\">\n      <meta name=\"topic-subtype\" content=\"MAXScript\">\n      <meta name=\"indexterm\" content=\"variables\">\n      <meta name=\"indexterm\" content=\"reference assignment\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Reference Assignment</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-FAF8B2F4-3199-4757-A604-49E1F5BBEBE9\"></a><div class=\"head\">\n            <h1>Reference Assignment </h1>\n         </div>\n         <div class=\"section\"><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AB0\"></a> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AB1\"></a>  MAXScript uses a type of assignment called <em class=\"mild\">reference assignment</em>. When you create a value, for example, by writing a literal, the memory for that\n               value is allocated and a pointer or <em class=\"mild\">reference</em> to the value is placed into the variable rather than the value itself. If you assign\n               that variable\'s value to another variable, the reference to that value\'s memory is\n               placed in the new variable. The same applies to assigning a variable’s value to an\n               array element or passing it as a function call argument. In all cases, the reference\n               is assigned or passed. When you assign a new value to a variable or when a variable\n               goes out of scope, the reference that the variable contained is dropped. When your\n               program has dropped all references to a value\'s memory, that memory becomes eligible\n               for reclamation. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AB4\"></a>This is in contrast to <em class=\"mild\">value assignment</em> in which a copy of the whole value\'s memory is made and assigned. Value assignment\n               has one advantage over reference assignment; every time a variable or an array element\n               is re-assigned, the old value’s memory space can be immediately reclaimed because\n               the old value is unique and nothing else can be referring to it. With reference assignment,\n               the system cannot reclaim the original value’s memory space because other variables\n               might still point to the same value. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AB5\"></a>Some scripting languages use a copy-based approach such as, HyperCard\'s HyperTalk,\n               but its inefficiency as well as some value sharing problems make it inappropriate\n               for MAXScript. This creates the task of reclaiming memory in MAXScript. Programming\n               languages such as, C or Pascal require that the programmer explicitly reclaim a value\'s\n               memory when it is no longer needed, and this is one of the most difficult parts of\n               programming in those languages. Recent variants like Java and some advanced languages\n               like Lisp and Smalltalk provide automatic memory reclamation. MAXScript also provides\n               this and it is described in <span class=\"char_link\"><a href=\"GUID-DF09BF99-2106-408D-9965-60436B853D76.htm\">Memory Allocation and Garbage Collection</a></span>. \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AB6\"></a>If you assign a variable\'s value to another variable, the reference to that value\'s\n               memory is placed in the new variable. Thus, both variables are actually referencing\n               the same value. If you assign a new value to one of the variables, the variable will\n               contain a reference to the new value and the other variable will contain a reference\n               to the old value. Normally this is the desired behavior as you do not want both variables\n               to reference the new value. However, a complication occurs if a compound value like\n               a point3 or array is referenced by more than one variable, and you assign a new value\n               to a component of the compound value. When you assign a value to a component of the\n               compound value, a new compound value is not created. Therefore, any variables that\n               reference the compound value will still point to the same value, and will \"see\" the\n               changed component value. Examples of this behavior are shown in the following script:\n               \n            </p> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86AB7\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"100%\">\n                  </colgroup>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86AB8\"></a><p class=\"table-body\"> <em class=\"strong\">SCRIPT</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-618F\"></a><pre>\n(\na= \"hello world\" -- create a string value, put reference in a\nb=a -- put string’s reference in b\nformat \"a=%; b=%\\n\" a b -- and print their values\nb= \"thanks for the fish\" -- create a string value, put reference in b\nformat \"a=%; b=%\\n\" a b -- print values - they are different\nb=a -- put string’s reference from a in b\na[1]= \"J\" -- change a component value\nformat \"a=%; b=%\\n\" a b -- print values - they are the same\na=b=[10,20,30] -- assign a point3 value reference to a and b\nformat \"a=%; b=%\\n\" a b -- and print their values\na.x=-100 -- change a component value\nformat \"a=%; b=%\\n\" a b -- print values - they are the same\n)\n</pre></div> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <a name=\"WS73099CC142F48755-2231E4B3128F27532E86AC9\"></a><p class=\"table-body\"> <em class=\"strong\">OUTPUT</em> \n                        </p> \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> \n                        <div class=\"codeBlock\"><a name=\"WS73099CC142F487553098682E12AC2FC2BC7-618E\"></a><pre>\na=hello world; b=hello world\na=hello world; b=thanks for the fish\na=Jello world; b=Jello world\na=[10,20,30]; b=[10,20,30]\na=[-100,20,30]; b=[-100,20,30]\nOK\n</pre></div> \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AD3\"></a>In some cases, this behavior is the desired behavior, and in other cases it is not.\n               For those cases where it is not the desired behavior, a <em class=\"strong\"> <span class=\"code\">copy()</span> </em> method is defined for most value types that will create a separate copy of a value.\n               \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AD4\"></a>In some cases, such as, arrays and structures, the value might contain compound values.\n               The copy made is called a <em class=\"mild\"> shallow copy</em>; only a copy of the upper-level value itself (that is, the array) is created. Copies\n               are not made of compound values in the value (that is, the elements of the array).\n               \n            </p> \n            <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AD5\"></a>In <em class=\"strong\">3ds Max 9 and higher</em>, a special copy method called <span class=\"code\"> <em class=\"strong\">deepCopy()</em> </span> is available for copying arrays. It allows the copying of all compound values inside\n               the array. See <span class=\"char_link\"><a href=\"GUID-A5B54C67-BFDD-45C0-9D6B-E6869817282A.htm\">Array Values</a></span> for details. \n            </p> \n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AD8\"></a> <span class=\"char_link\"><a href=\"GUID-CA7ABB4B-6CB6-47B7-903D-84D2F45D993D.htm\">Manual Garbage Collection</a></span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F48755-2231E4B3128F27532E86AD9\"></a> <span class=\"char_link\"><a href=\"GUID-DF09BF99-2106-408D-9965-60436B853D76.htm\">Memory Allocation and Garbage Collection</a></span> \n                     </p> \n                  </li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";